badge <- function(data, cc.var, exposure.var, group.var, out.dir, op=NULL) {
 
  op <- default.list(op, 
         c("alpha_min", "alpha_max", "beta_min", "beta_max",
           "sigma2_beta", "sigma2_alpha", "sigma2_psi",
           "psi_k", "psi_theta", "psi_min", "psi_max",
           "w_m", "update_prop", "n_iter", "n_sep_out",
           "random_seed", "cluster_size", "k_max",
           "method_psi", "tao_min", "tao_max", "sigma2_tao",
           "split_option", "shape_split_a", "shape_split_b",
           "shape_merge_a", "shape_merge_b", "update_prop_w",
           "out.string",
           "method_alpha", "method_beta", "method_tao",
           "alpha_mu", "alpha_sigma2", "beta_mu", "beta_sigma2",
           "tao_mu", "tao_sigma2",
           "method"),
          list(-4, 4, -4, 4,
               0.005, 0.005, 0.02,
               4.0, 0.2, 0.0, 1.2,
               50, 1.0, 200000, 1,
               12345, 0.0, 2, 
               0, -4, 4, 0.005,
               0, 2, 2, 
               2, 2, 1,
               "",
               0, 0, 0,
               0, 4, 0, 4,
               0, 4,
               0
               )
          )
  op$num_of_cluster <- op$k_max

  if (!is.data.frame(data)) stop("ERROR: data must be a data frame")
  if (is.null(cc.var)) stop("ERROR: cc.var must be specified")
  if (is.null(exposure.var)) stop("ERROR: exposure.var must be specified")
  if (is.null(group.var)) stop("ERROR: group.var must be specified")
  if (is.null(out.dir)) stop("ERROR: out.dir must be specified")

  method <- op$method
  if (!(method %in% 0:1)) stop("ERROR: method must be 0 or 1")
  op$method_alpha <- method
  op$method_beta  <- method
  op$method_tao   <- method

  covars     <- op[["covars", exact=TRUE]]
  ncovars    <- length(covars)
  covarsFlag <- !is.null(covars)
  out.dir    <- checkForSep(out.dir)
  sim.mat    <- op[["sim.mat", exact=TRUE]]
  simFlag    <- !is.null(sim.mat)

  # Check the length of the tao parameters
  if (ncovars) {
    op$tao_min <- paste(rep(op$tao_min, ncovars)[1:ncovars], collapse=",", sep="")
    op$tao_max <- paste(rep(op$tao_max, ncovars)[1:ncovars], collapse=",", sep="")
    op$tao_mu  <- paste(rep(op$tao_mu, ncovars)[1:ncovars], collapse=",", sep="")
    op$tao_sigma2 <- paste(rep(op$tao_sigma2, ncovars)[1:ncovars], collapse=",", sep="")
  }

  # Check variable names
  vlist <- list(response.var=cc.var, snp.var=exposure.var, main.vars=covars,
               group.var=group.var)
  vars <- getAllVars(vlist, names=names(vlist))
  temp <- !(vars %in% colnames(data))
  if (any(temp)) {
    print(vars[temp])
    stop("The above variables were not found in the input data")
  }

  # Check the variables
  if (!all(data[, cc.var] %in% 0:1)) stop("ERROR: cc.var must be coded as 0-1")
  if (!is.numeric(data[, exposure.var])) stop("ERROR: exposure.var must be numeric")

  # Check the group variable 
  temp   <- data[, group.var]
  ngroup <- length(unique(temp))
  if (!all(temp %in% 1:ngroup)) stop("ERROR: group.var must be coded as 1, 2, 3, ...")
  data[, group.var] <- as.numeric(temp) - 1

  # Check the similarity matrix 
  if (simFlag) {
    sim.mat <- as.matrix(sim.mat)
    if (nrow(sim.mat) != ncol(sim.mat)) stop("ERROR: sim.mat must be a square matrix")
    if (any(is.na(sim.mat))) stop("ERROR: sim.mat cannot contain missing values")
    if (max(abs(sim.mat - t(sim.mat))) > 1e-5) stop("ERROR: sim.mat must be symmetric")
    if (ngroup != nrow(sim.mat)) stop("ERROR: dimension of sim.mat is not correct")
    if (any(diag(sim.mat) != 0)) stop("ERROR: diagonal of sim.mat should be 0")
  } else {
    sim.mat <- matrix(data=0, nrow=ngroup, ncol=ngroup)
  }

  # Check for missing values
  nsub <- nrow(data)
  temp <- getFormulas(vlist)
  miss <- c(NA, NaN, Inf, -Inf)
  if (length(temp)) data <- applyFormulas(data, temp, remove=miss)
  data <- removeMiss.vars(data, vars=vars, miss=miss)
  if (nsub != nrow(data)) stop("ERROR: data must not contain missing values")

  # Get the design matrix
  if (covarsFlag) {
    design <- removeOrKeepCols(data, covars)
  } else {
    design <- NULL
  }

  # Create file names
  str <- op$out.string
  data.file <- paste(out.dir, str, "_TEMP_data.txt", sep="")
  sim.file  <- paste(out.dir, str, "_TEMP_sim.txt", sep="")
  parm.file <- paste(out.dir, str, "_TEMP_parm.txt", sep="")

  # For psi
  method_psi <- op$method_psi
  rm_psi     <- 0
  if (!(method_psi %in% 0:2)) stop("ERROR: method_psi must be 0, 1, or 2")
  if (method_psi == 2) {
    psi_file <- op[["psi_file", exact=TRUE]]
    if (is.null(psi_file)) {
      rm_psi <- 1
      
      stop("ERROR: psi_file must be specified for method_psi = 2")
    }
  } else {
    op$psi_file <- "NA"
  }

  # Create the parameter file
  names <- c("alpha_min", "alpha_max", "beta_min", "beta_max",
           "sigma2_beta", "sigma2_alpha", "sigma2_psi",
           "psi_k", "psi_theta", "psi_min", "psi_max",
           "w_m", "update_prop", "n_iter", "n_sep_out",
           "random_seed", "cluster_size", "k_max",
           "method_psi", "tao_min", "tao_max", "sigma2_tao",
           "split_option", "shape_split_a", "shape_split_b",
           "shape_merge_a", "shape_merge_b", "num_of_cluster", "update_prop_w",
           "method_alpha", "method_beta", "method_tao",
           "alpha_mu", "alpha_sigma2", "beta_mu", "beta_sigma2",
           "tao_mu", "tao_sigma2", "psi_file")
  out <- c("out_k", "out_psi", "out_alpha", "out_beta", "out_alloc_z",
           "out_dev_i", "out_tao")
  n <- length(names) + length(out) + 2
  x <- character(n)
  x[1] <- "NAME,VALUE"
  x[2] <- paste("data,", data.file, sep="")
  x[3] <- paste("sim_matrix,", sim.file, sep="")
  index <- 4
  for (nn in out) {
    temp <- paste(out.dir, str, nn, ".txt", sep="")
    x[index] <- paste(nn, ",", temp, sep="")
    index <- index + 1
  }
  for (nn in names) {
    temp <- op[[nn, exact=TRUE]]
    x[index] <- paste(nn, ",", temp, sep="")
    index <- index + 1
  }
  write(x, file=parm.file, ncolumns=1)

  # Write the similarity matrix
  write.table(sim.mat, file=sim.file, col.names=FALSE, sep=" ", row.names=FALSE, quote=FALSE)

  # Write out the data file
  data <- cbind(data[, cc.var], data[, exposure.var], data[, group.var], design)
  write.table(data, file=data.file, col.names=FALSE, sep=" ", row.names=FALSE, quote=FALSE)
  argc <- 2
  ret  <- 1
  if (op$random_seed > 0) set.seed(op$random_seed)
  temp <- .C("trans", as.integer(argc), parm.file, ret=as.integer(ret), PACKAGE="BaDGE")
  if (temp$ret) {
    stop("ERROR: calling the trans C function")
  }
  
  # Delete temporary files
  file.remove(c(data.file, sim.file, parm.file))
 
  NULL         
}